/*
    PersistentPreferences.hpp

    NOTE: This file is a generated file.
          Do not modify it by hand!
*/

#ifndef PersistentPreferences_hpp_included
#define PersistentPreferences_hpp_included


/***********************************************************************************************************************

Name   : PersistentPreferences Component

Author : Steffen Uhlig <Steffen.Uhlig@htwm.de>
       : Andr Schild  <ASchild@neatech.ch>

Remarks: This native Power++ 2.1 component stores window placement, numbers, strings and LRU items in the registry.

         By default the location "HKEY_CURRENT_USER\Software\CompanyName\ProductName\Version\" will be used, where 
         "CompanyName", "ProductName" and "Version" will be automatically retrieved from the executable using this 
         component.

         Registry leaf (e.g. HKEY_CURRENT_USER) and location ("Software") may be overwritten at design time by 
         specifying other strings in the component's property sheet, the former and "CompanyName", "ProductName" and
         "Version" may also be specified at run time by using the appropriate member functions.

         This component uses the VersionInfo component by Andre Schild <ASchild@neatech.ch>.

         To see how the "PersistentPreferences" component may be used please see also the included demonstration 
         project "PPdemo". It is currently bound to this project (see Run - Run Options - DLL).

TODO:    - currently there is no support for storing signed and floating point numbers

NEW:     The complete LRU management has been added by A.Schild  (1998.06.25)


Bugs /  Comments
----------------

If you have trouble recompiling this project be sure to have the VersionInfo component installed. You may also have to 
switch off the "Create install image ..." in the run options dialog for the designtime DLL (or adjust the path).

If you want to submit a bug or if you wish to contribute to the further development of the component please contact the
author by eMail (address see above). Any comments about the component are welcome (especially if you have a better or 
more descriptive name for the component :-).

The legal stuff
---------------

THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, EXPRESSED, IMPLIED OR OTHERWISE, INCLUDING AND 
WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL THE AUTHOR
BE LIABLE FOR ANY SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES WHATSOEVER (INCLUDING, WITHOUT LIMITATION, 
DAMAGES FOR LOSS OF PROFITS, BUSINESS INTERRUPTION, LOSS OF INFORMATION, OR ANY OTHER LOSS) , WHETHER OR NOT ADVISED 
OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR INABILITY
TO USE THIS SOFTWARE.

Steffen Uhlig (8.2.98)
Andr Schild  (25.6.98)
Andr Schild  (10.4.2001)


The "Advertisement" Text
------------------------

Do you want to restore the position, size and state of your application's form(s) at program startup (like the Explorer
windows and many other programs)? Do you store this information in the registry? Have you ever been annoyed by writing
the same "writeXXX" and "readXXX" functions in every project over and over again, just to save numbers and strings at 
the same place?

Well, here comes the solution: the native PersistentPreferences component. It automatically creates the appropriate 
registry key from the version resources of your project and you can store everything in a single place. You only need
a single line of code to store the window placement, a string or a number in the registry:

    pp_1->SaveWindowPosition(this); // example to store the complete window placement of the current form
    pp_1->SetNumber("MyNumber", _myNumber); // store the value of _myNumber under the name "MyNumber"

The restore / retrieve is as simple as

    pp_1->RestoreWindowPosition(this); // restore the complete window placement of the current form
    WString myString = pp_1->GetString("myString"); // get the string stored under "myString"

Usually the Strings and numbers are stored in a subkey, including the version number.
But usually some settings are version-independent. For those cases, just ause the Set/GetStringVI 
methods.

The LRU Management is simple:

1. Create a Menubar for the form
2. Create a Formmethode in the form WBool LRUClicked(WObject *source,WEventData *event );
   This is the eventhandler for the Click event of the LRU entries
   With ((WMenuItem *)source)->GetText().Substring(4) you can get the LRU string selected in the menu
3. In the create event of the form connect the menu with the LRU handler
   pp_1->SetLRUMenu(menu_1_Item0, WEventHandlerCast(Form1, LRUClicked));
   This will reload the saved LRU items from the registry and insert them in the 
   menu. The first parameter specifies the menuitem where it is included.
4. At anytime you wish, you can call AddLRU methode to add a new entry in the LRU menu.
   It is the saved in the registry

Have a look at the included ppdemo


By default the registry location where all entries are stored follows the windows UI guidelines (HKEY_CURRENT_USER - 
Software - CompanyName - ProductName - Version), but the component user is free to change this behaviour.

Requirements: Powersoft Power++ 2.0, VersionInfo component

Please look at Bill Klein's "Absolute Power++" site (http://www.orbit.org/power/) for the latest release.

Author: Steffen Uhlig <Steffen.Uhlig@htwm.de>

***********************************************************************************************************************/

// Declarations added here will be included at the top of the .HPP file

// by default the folder "$(__OptimaDir)\components" is part of the libraries path
#include "VerInfo\VersionInfo.hpp"

typedef WArray<WMenuItem *> WMenuItemPtrArray;
#ifdef __OPTIMA_USE_COMPONENT_LIBRARIES__
#  pragma library("PersistentPreferencesComponent/PersistentPreferencesComponent.lib")
#endif // __OPTIMA_USE_COMPONENT_LIBRARIES__

#if !defined( __PersistentPreferences_declspec )
#define __PersistentPreferences_declspec
#endif

class __PersistentPreferences_declspec PersistentPreferences : public WObject
{
    public:
    public:
        // add your public instance data here
    private:
        VersionInfo _versionInfo;
        WRegistryKey _key;
    protected:
        // add your protected instance data here
    private:
        WString _companyName;            // store CompanyName Property
        WString _productVersion;         // store Version Property
        WString _productName;            // store ProductName Property
        WRegistryKeyHandle _registryKeyHandle;      // store RegistryKeyHandle Property
        WString _subKey;                 // store SubKey Property
    private:
        WInt _LRUEntries;                // store LRUEntries Property
    private:
        WBool _LRUCaseSensitive;         // store LRUCaseSensitive Property
    private:
        WInt _LRUMenuItemBelow;             // store LRUMenuItemBelow Property
    private:
        WBool _LRUMenuSeparatorAbove;       // store LRUMenuSeparatorAbove Property
    private:
        WBool _LRUMenuSeparatorBelow;       // store LRUMenuSeparatorBelow Property
    private:
        WPopupMenu * _LRUMenu;              // store LRUMenu Property
        WMenuItemPtrArray   _LRUMenuItems;  // stores the LRUMenuItems
        WEventHandler       _LRUClickHandler;  // eventhandler for LRU click events
    public:
        //
        // CompanyName Property
        //
        WString GetCompanyName() const { return _companyName; }
        WBool   SetCompanyName( WString const & __newValue ) { _companyName = __newValue; return TRUE; }
    public:
        //
        // ProductVersion Property
        //
        WString GetProductVersion() const { return _productVersion; }
        WBool   SetProductVersion( WString const & __newValue ) { _productVersion = __newValue; return TRUE; }
    public:
        //
        // ProductName Property
        //
        WString GetProductName() const { return _productName; }
        WBool   SetProductName( WString const & __newValue ) { _productName = __newValue; return TRUE; }
    public:
        //
        // RegistryKeyHandle Property
        //
        WRegistryKeyHandle GetRegistryKeyHandle() const { return _registryKeyHandle; }
        WBool              SetRegistryKeyHandle( WRegistryKeyHandle const & __newValue ) { _registryKeyHandle = __newValue; return TRUE; }
    public:
        //
        // SubKey Property
        //
        WString GetSubKey() const { return _subKey; }
        WBool   SetSubKey( WString const & __newValue ) { _subKey = __newValue; return TRUE; }
    public:
        //
        // String Property
        //
        WString GetString(WString valueName);
        WBool SetString(WString valueName, WString value);
    public:
        //
        // Number Property
        //
        WDWord GetNumber(WString valueName);
        WBool SetNumber(WString valueName, WDWord newValue);
    public:
        //
        // WindowPosition Property
        //
        WBool RestoreWindowPosition(WWindow * toRestore, WString valueName = "WindowPlacement");
        WBool SaveWindowPosition(WWindow * toSave, WString valueName = "WindowPlacement");
    public:
        //
        // LRUEntries Property
        //

        WInt  GetLRUEntries() const { return _LRUEntries; }
        WBool SetLRUEntries( WInt const & __newValue );
    public:
        //
        // LRUText Property
        //

        WString GetLRUText(const WInt entry);
        WBool SetLRUText( const WInt entry, const WString &lruEntry);
    public:
        //
        // LRUCaseSensitive Property
        //

        WBool GetLRUCaseSensitive() const { return _LRUCaseSensitive; }
        WBool SetLRUCaseSensitive( WBool const & __newValue ) { _LRUCaseSensitive = __newValue; return TRUE; }
    public:
        //
        // LRUMenuItemBelow Property
        //

        WInt  GetLRUMenuItemBelow() const { return _LRUMenuItemBelow; }
        WBool SetLRUMenuItemBelow( WInt const & __newValue ) { _LRUMenuItemBelow = __newValue; return TRUE; }
    public:
        //
        // LRUMenuSeparatorAbove Property
        //

        WBool GetLRUMenuSeparatorAbove() const { return _LRUMenuSeparatorAbove; }
        WBool SetLRUMenuSeparatorAbove( WBool const & __newValue ) { _LRUMenuSeparatorAbove = __newValue; return TRUE; }
    public:
        //
        // LRUMenuSeparatorBelow Property
        //

        WBool GetLRUMenuSeparatorBelow() const { return _LRUMenuSeparatorBelow; }
        WBool SetLRUMenuSeparatorBelow( WBool const & __newValue ) { _LRUMenuSeparatorBelow = __newValue; return TRUE; }
    public:
        //
        // LRUMenu Property
        //

        WPopupMenu * GetLRUMenu() const { return _LRUMenu; }
        WBool   SetLRUMenu( WPopupMenu * __newValue, WEventHandler clickHandler );
        WBool   SetLRUMenu( WPopupMenu * __newValue, WEventHandler clickHandler, const WInt __LRUMenuItemBelow);
        WBool   SetLRUMenu( WPopupMenu * __newValue, WEventHandler clickHandler, const WInt __LRUMenuItemBelow, const _LRUMenuSeparatorAbove , const _LRUMenuSeparatorBelow = false);
    public:
        //
        // StringVI Property
        //

        WString GetStringVI(WString valueName) ;
        WBool SetStringVI(WString valueName, WString newValue );
    public:
        //
        // NumberVI Property
        //

        WDWord GetNumberVI(WString valueName) ;
        WBool SetNumberVI(WString valueName, WDWord newValue);
    public:
        //
        // AddLRU Method
        //

        WInt AddLRU(const WString &fName);
    public:
        PersistentPreferences();
    public:
        ~PersistentPreferences();
    private:
        WBool _openKey(void);
    private:
        WBool _closeKey(void);
    protected:
        void DisplayLRU();
    protected:
        void HideLRU( void );
    private:
        WBool _openKeyVI(void);
};

#endif
